==> 1.3.6.tar.gz: lib/src/live_data_mutable.dart <==
part of '../flutterx_live_data.dart';

class MutableLiveData<T> extends LiveData<T> {
  MutableLiveData({T? initialValue}) : assert(initialValue is T, 'initialValue must be of type $T, got $initialValue') {
    setData(initialValue as T);
  }

  MutableLiveData.late();

  set value(T value) => setData(value);

  void postValue(FutureOr<T> value) => postData(value);
}

class MutableMediatorLiveData<T> extends MutableLiveData<T> with MediatorMixin<T> {
  MutableMediatorLiveData({T? initialValue}) : super(initialValue: initialValue);

  MutableMediatorLiveData.late() : super.late();
}

class HistoryMutableLiveData<T> extends MutableLiveData<T> with HistoryMixin<T> {
  HistoryMutableLiveData({T? initialValue}) : super(initialValue: initialValue);

  HistoryMutableLiveData.late() : super.late();
}

class StorageMutableLiveData<T> extends MutableLiveData<T> with StorageMixin<T> {
  @override
  final String key;

  @override
  final T defaultValue;

  @override
  final Map<String, String> _storage;

  @override
  final String Function(T value) _encode;

  StorageMutableLiveData({
    required this.key,
    required Map<String, String> storage,
    T? defaultValue,
    String Function(T value) encode = _defaultEncode,
    T Function(String source)? decode,
  })  : assert(defaultValue is T, 'defaultValue must be of type $T, got $defaultValue'),
        _storage = storage,
        _encode = encode,
        defaultValue = defaultValue as T,
        super(initialValue: StorageMixin.resolve(key, storage, defaultValue, decode ?? _defaultDecode));

  StorageMutableLiveData.enumeration({
    required String key,
    required Map<String, String> storage,
    T? defaultValue,
    required Iterable<T> values,
  }) : this(
            key: key,
            storage: storage,
            defaultValue: defaultValue,
            encode: StorageMixin.encodeEnum,
            decode: (name) => StorageMixin.decodeEnum(name, values));

  StorageMutableLiveData.json({
    required String key,
    required Map<String, String> storage,
    T? defaultValue,
    dynamic Function(T value)? toJson,
    required T Function(dynamic json) fromJson,
  }) : this(
            key: key,
            storage: storage,
            defaultValue: defaultValue,
            encode: (value) => jsonEncode(toJson == null ? value : toJson(value)),
            decode: (source) => fromJson(jsonDecode(source)));

  StorageMutableLiveData.jsonObject({
    required String key,
    required Map<String, String> storage,
    T? defaultValue,
    dynamic Function(T value)? toJson,
    required T Function(Map<String, dynamic> json) fromJson,
  }) : this(
            key: key,
            storage: storage,
            defaultValue: defaultValue,
            encode: (value) => jsonEncode(toJson == null ? value : toJson(value)),
            decode: (source) => fromJson(jsonDecode(source)));

  StorageMutableLiveData.int({
    required String key,
    required Map<String, String> storage,
    T? defaultValue,
  }) : this(
            key: key,
            storage: storage,
            defaultValue: defaultValue,
            decode: (source) => source == 'null' ? null as T : int.parse(source) as T);

  StorageMutableLiveData.double({
    required String key,
    required Map<String, String> storage,
    T? defaultValue,
  }) : this(
            key: key,
            storage: storage,
            defaultValue: defaultValue,
            decode: (source) => source == 'null' ? null as T : double.parse(source) as T);

  StorageMutableLiveData.bool({
    required String key,
    required Map<String, String> storage,
    T? defaultValue,
  }) : this(
            key: key,
            storage: storage,
            defaultValue: defaultValue,
            decode: (source) => source == 'null' ? null as T : ('true' == source) as T);

  static String _defaultEncode(dynamic value) => value.toString();

  static T _defaultDecode<T>(String source) {
    assert(source is T,
        'Default decode only supports ${source.runtimeType}, got $T. Use a custom decode function instead');
    return source as T;
  }
}

==> 1.4.0.tar.gz: lib/src/live_data_mutable.dart <==
part of '../flutterx_live_data.dart';

/// A [LiveData] instance which exposes [value] setter and [postValue] method
class MutableLiveData<T> extends LiveData<T> {
  /// Create an instance with an [initialValue].
  /// Default value is null, but if T is not nullable this will produce an exception
  MutableLiveData({T? initialValue}) : assert(initialValue is T, 'initialValue must be of type $T, got $initialValue') {
    setData(initialValue as T);
  }

  /// Create an uninitialized instance
  MutableLiveData.late();

  set value(T value) => setData(value);

  /// Exposes [postData]
  void postValue(FutureOr<T> value) => postData(value);
}

/// [MediatorMixin] implementation of [MutableLiveData]
class MutableMediatorLiveData<T> extends MutableLiveData<T> with MediatorMixin<T> {
  @override
  final bool reactive;

  /// Check [MutableLiveData.new]
  MutableMediatorLiveData({T? initialValue, this.reactive = true}) : super(initialValue: initialValue);

  /// Check [MutableLiveData.late]
  MutableMediatorLiveData.late({this.reactive = true}) : super.late();
}

/// [HistoryMixin] implementation of [MutableLiveData]
class HistoryMutableLiveData<T> extends MutableLiveData<T> with HistoryMixin<T> {
  /// Check [MutableLiveData.new]
  HistoryMutableLiveData({T? initialValue}) : super(initialValue: initialValue);

  /// Check [MutableLiveData.late]
  HistoryMutableLiveData.late() : super.late();
}

/// [StorageMixin] implementation of [MutableLiveData]
class StorageMutableLiveData<T> extends MutableLiveData<T> with StorageMixin<T> {
  @override
  final String key;

  @override
  final T defaultValue;

  @override
  final Map<String, String> _storage;

  @override
  final String Function(T value) _encode;

  /// Check [StorageMixin]
  StorageMutableLiveData({
    required this.key,
    required Map<String, String> storage,
    T? defaultValue,
    String Function(T value) encode = StorageMixin.defaultEncode,
    T Function(String source)? decode,
  })  : assert(defaultValue is T, 'defaultValue must be of type $T, got $defaultValue'),
        _storage = storage,
        _encode = encode,
        defaultValue = defaultValue as T,
        super(initialValue: StorageMixin.resolve(key, storage, defaultValue, decode ?? StorageMixin.defaultDecode));

  /// Check [StorageMixin]
  StorageMutableLiveData.int({
    required String key,
    required Map<String, String> storage,
    T? defaultValue,
  }) : this(
            key: key,
            storage: storage,
            defaultValue: defaultValue,
            decode: (source) => source.isEmpty ? null as T : int.parse(source) as T);

  /// Check [StorageMixin]
  StorageMutableLiveData.double({
    required String key,
    required Map<String, String> storage,
    T? defaultValue,
  }) : this(
            key: key,
            storage: storage,
            defaultValue: defaultValue,
            decode: (source) => source.isEmpty ? null as T : double.parse(source) as T);

  /// Check [StorageMixin]
  StorageMutableLiveData.bool({
    required String key,
    required Map<String, String> storage,
    T? defaultValue,
  }) : this(
            key: key,
            storage: storage,
            defaultValue: defaultValue,
            decode: (source) => source.isEmpty ? null as T : ('true' == source) as T);

  /// Check [StorageMixin]
  StorageMutableLiveData.enumeration({
    required String key,
    required Map<String, String> storage,
    T? defaultValue,
    required Iterable<T> values,
  }) : this(
            key: key,
            storage: storage,
            defaultValue: defaultValue,
            encode: StorageMixin.encodeEnum,
            decode: (source) => StorageMixin.decodeEnum(source, values));

  /// Check [StorageMixin]
  StorageMutableLiveData.jsonObject({
    required String key,
    required Map<String, String> storage,
    T? defaultValue,
    dynamic Function(T value)? toJson,
    required T Function(Map<String, dynamic> json) fromJson,
  }) : this(
            key: key,
            storage: storage,
            defaultValue: defaultValue,
            encode: (value) => jsonEncode(toJson == null ? value : toJson(value)),
            decode: (source) => fromJson(jsonDecode(source)));

  /// Check [StorageMixin]
  StorageMutableLiveData.jsonList({
    required String key,
    required Map<String, String> storage,
    T? defaultValue,
    dynamic Function(T value)? toJson,
    required T Function(List json) fromJson,
  }) : this(
            key: key,
            storage: storage,
            defaultValue: defaultValue,
            encode: (value) => jsonEncode(toJson == null ? value : toJson(value)),
            decode: (source) => fromJson(jsonDecode(source)));
}
